#
# Sonar, entreprise quality control tool.
# Copyright (C) 2007-2008 Hortis-GRC SA
# mailto:be_agile HAT hortis DOT ch
#
# Sonar is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 3 of the License, or (at your option) any later version.
#
# Sonar is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with Sonar; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
#
class Sonar::DashboardConfiguration

  COLUMN_SEPARATOR = '.'
  COLUMNS_SEPARATOR = ';'

  COLUMNS_SELECTED_KEY = 'sonar.core.projectsdashboard.columns'
  COLUMNS_DEFAULT_SORT_KEY = 'sonar.core.projectsdashboard.defaultSortedColumn'
  TREEMAP_ENABLED_KEY = 'sonar.core.projectsdashboard.showTreemap'


  def initialize(configuration)
    @configuration = configuration
  end

  def displayed_columns
    columns_from_text(get_text_columns)
  end

  def homepage_metrics(treemap_metrics)
    columns = displayed_columns
    metrics = columns.select{|col| col.metric_column?}.collect{|col| Metric.by_name(col.id)}
    metrics.concat(treemap_metrics)
    metrics<<Metric.by_name(Metric::ALERT_STATUS)
    metrics.uniq.compact
  end

  def addeable_columns
    addeable_columns = available_columns
    configured = displayed_columns
    addeable_columns.each_key { |domain|
      domain_addeable_columns = addeable_columns[domain]
      domain_addeable_columns.delete_if { |columns_view|
        deleteable = false
        configured.each do |configured_column|
          deleteable = true if columns_view.name == configured_column.name
        end
        deleteable
      }
    }
    addeable_columns
  end

  def get_selected_columns
    columns_from_text(get_text_columns)
  end

  def find_selected_column(column_id)
    get_selected_columns.detect {|column| column.id == column_id}
  end

  def add_column(column)
    columns = get_selected_columns << column
    @configuration.setProperty(COLUMNS_SELECTED_KEY, columns_to_text(columns))
  end

  def remove_column(column)
    columns = get_selected_columns.reject {|col| col.id == column.id}
    @configuration.setProperty(COLUMNS_SELECTED_KEY, columns_to_text(columns))
  end

  def move_column(column, direction)
    position = column.position
    columns = get_selected_columns.reject {|col| col.id == column.id}
    new_position = (direction == "left") ? position - 1 : position + 1
    columns = columns.insert(new_position, column)
    @configuration.setProperty(COLUMNS_SELECTED_KEY, columns_to_text(columns))
  end

  def find_available_column(column_id)
    available_columns.each_pair do |domain, columns|
      columns.each do |column|
        if column.id == column_id
          return column
        end
      end
    end
    nil
  end

  def get_text_column_sort_default
    @configuration.getString(COLUMNS_DEFAULT_SORT_KEY, "build_time")
  end

  def set_column_sort_default(column_id)
    @configuration.setProperty(COLUMNS_DEFAULT_SORT_KEY, column_id)
  end

  def is_treemap_enabled?
    @configuration.getBoolean(TREEMAP_ENABLED_KEY, true)
  end

  def toggle_treemap_enabled
    @configuration.setProperty(TREEMAP_ENABLED_KEY, !is_treemap_enabled?)
  end


  @@available_columns = nil

  DEFAULT_DOMAIN='General'

  def available_columns
    if @@available_columns.nil?
      @@available_columns = {}

      @@available_columns[DEFAULT_DOMAIN] = []
      col = Sonar::ColumnsView.new
      col.name = "Project"
      col.id = 'project'
      col.col_type = Sonar::ColumnsView::TYPE_PROJECT
      @@available_columns[DEFAULT_DOMAIN] << col


      col = Sonar::ColumnsView.new
      col.name = "Links"
      col.id = 'links'
      col.col_type = Sonar::ColumnsView::TYPE_LINKS
      @@available_columns[DEFAULT_DOMAIN] << col

      col = Sonar::ColumnsView.new
      col.name = "Build time"
      col.id = 'build_time'
      col.col_type = Sonar::ColumnsView::TYPE_BUILD_TIME
      @@available_columns[DEFAULT_DOMAIN] << col

      col = Sonar::ColumnsView.new
      col.name = "Language"
      col.id = 'language'
      col.col_type = Sonar::ColumnsView::TYPE_LANGUAGE
      @@available_columns[DEFAULT_DOMAIN] << col

      col = Sonar::ColumnsView.new
      col.name = "Version"
      col.id = 'version'
      col.col_type = Sonar::ColumnsView::TYPE_VERSION
      @@available_columns[DEFAULT_DOMAIN] << col
      
      Metric.all.select {|m| m.display?}.each do |metric|
        col = Sonar::ColumnsView.new
        col.name = metric.short_name
        col.id = metric.name
        col.col_type = Sonar::ColumnsView::TYPE_METRIC

        if col.name
          if metric.domain
            @@available_columns[metric.domain] ||= []
            @@available_columns[metric.domain] << col
          else
            @@available_columns[DEFAULT_DOMAIN] << col
          end
        end
      end

      categories_domain = Metric::DOMAIN_RULES + ' categories'
      @@available_columns[categories_domain] = []
      RulesCategory.all.each do |rules_category|
        col = Sonar::ColumnsView.new
        col.name = rules_category.name
        col.id = rules_category.name
        col.col_type = Sonar::ColumnsView::TYPE_RULES_CATEGORY

        @@available_columns[categories_domain] << col
      end

      @@available_columns.each_value {|columns|
        columns.sort! { |x, y|
          x.name = "" if x.name.nil?
          y.name = "" if y.name.nil?
          x.name <=> y.name if x.name and y.name
        }
      }

    end
    # must return a copy of the hash !
    available_columns_clone = {}
    @@available_columns.each_pair {|domain, columns|
      available_columns_clone[domain] = columns.clone
    }
    available_columns_clone
  end


  protected

  def get_text_columns
    @configuration.getString(COLUMNS_SELECTED_KEY, default_selected_columns)
  end

  def default_selected_columns
    columns = ""
    columns << Sonar::ColumnsView::TYPE_PROJECT + COLUMN_SEPARATOR + "project" + COLUMNS_SEPARATOR
    columns << Sonar::ColumnsView::TYPE_METRIC + COLUMN_SEPARATOR + Metric::RULES_MANDATORY_INDEX + COLUMNS_SEPARATOR
    columns << Sonar::ColumnsView::TYPE_METRIC + COLUMN_SEPARATOR + Metric::CODE_COVERAGE + COLUMNS_SEPARATOR
    columns << Sonar::ColumnsView::TYPE_BUILD_TIME + COLUMN_SEPARATOR + 'build_time' + COLUMNS_SEPARATOR
    columns << Sonar::ColumnsView::TYPE_LINKS + COLUMN_SEPARATOR + "links"
    columns
  end

  def columns_from_text(text)
    columns = []
    text.split(COLUMNS_SEPARATOR).each_with_index do |column_text, position|
      column = column_from_text(column_text)
      column.position = position
      column.sort_default = true if column.id == get_text_column_sort_default
      columns << column
    end
    columns
  end

  def column_from_text(text)
    column_split = text.split(COLUMN_SEPARATOR)
    column = Sonar::ColumnsView.new
    column.id = column_split[1]
    column.col_type = column_split[0]
    column.name = find_available_column(column.id).name
    column
  end

  def columns_to_text(columns)
    text = ""
    columns.each do |column|
      text << column_to_text(column) + COLUMNS_SEPARATOR
    end
    text
  end

  def column_to_text(column)
    text = column.col_type + COLUMN_SEPARATOR + column.id
  end

end
